@@ -98,10 +98,12 @@ def output(directory=None):
 
 def tfimport(addr, id, directory=None):
     logger.info(f"terraform import directory={directory} addr={addr} id={id}")
+
     command = ["import", addr, id]
+
     with timer(logger, "terraform import"):
         run_terraform_subprocess(
-            command, cwd=directory, prefix="terraform", strip_errors=True
+            command, cwd=directory, prefix="terraform", strip_errors=True, timeout=30
         )
 
 
